/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Fabian 'x3n' Landau
 *   Co-authors:
 *      Dominik Solenicki
 *
 */
#include "controllers/CommonController.h"

namespace orxonox
{

    RegisterClass(CommonController);
    static const float SPEED = 0.6f;
    static const float ROTATEFACTOR = 0.2f;

    /*static const float SPEED_FREE = 0.8f;
    static const float ROTATEFACTOR_FREE = 0.8f;*/
    
    bool CommonController::setWingman (CommonController* wingman)
    {
        return false;
    }
    bool CommonController::isLeader ()
    {
        return false;
    }
    bool CommonController::hasWingman()
    {
        return true;
    }

    CommonController::CommonController(Context* context) : Controller(context)
    {

        RegisterObject(CommonController);
    }


    CommonController::~CommonController()
    {
    }
    //copy the Roll orientation of given Quaternion.
    void CommonController::copyOrientation(const Quaternion& orient)
    {
        //roll angle difference in radian
        float diff=orient.getRoll(false).valueRadians()-(this->getControllableEntity()->getOrientation().getRoll(false).valueRadians());
        while(diff>math::twoPi) diff-=math::twoPi;
        while(diff<-math::twoPi) diff+=math::twoPi;
        this->getControllableEntity()->rotateRoll(-diff);



    }
    void CommonController::setTargetPosition(const Vector3& target)
    {
        this->targetPosition_ = target;
        this->bHasTargetPosition_ = true;
    }

    void CommonController::copyTargetOrientation()
    {
        if (bHasTargetOrientation_)
        {   
            copyOrientation(targetOrientation_);
        }
    }
    void CommonController::setTargetOrientation(const Quaternion& orient)
    {
        this->targetOrientation_=orient;
        this->bHasTargetOrientation_=true;
    }
    void CommonController::setTargetOrientation(ControllableEntity* target)
    {
        if (target)
            setTargetOrientation(target->getOrientation());
    }


    void CommonController::moveToPosition(const Vector3& target)
    {
        if (!this->getControllableEntity())
            return;
        
        //100 is (so far) the smallest tolerance (empirically found) that can be reached, 
        //with smaller distance spaceships can't reach position and go circles around it instead
        int tolerance = 100;

        ControllableEntity* entity = this->getControllableEntity();
        Vector2 coord = get2DViewCoordinates
            (entity->getPosition(), 
            entity->getOrientation() * WorldEntity::FRONT, 
            entity->getOrientation() * WorldEntity::UP, 
            target);

        float distance = (target - this->getControllableEntity()->getPosition()).length();

        //rotates should be in range [-1,+1], clamp cuts off all that is not
        float rotateX = clamp(coord.x * 10, -1.0f, 1.0f);
        float rotateY = clamp(coord.y * 10, -1.0f, 1.0f);

        
        if (distance > tolerance)
        {
            //Yaw and Pitch are enough to start facing the target
            this->getControllableEntity()->rotateYaw(-2.0f * ROTATEFACTOR * rotateX);
            this->getControllableEntity()->rotatePitch(2.0f * ROTATEFACTOR * rotateY);

            //300 works, maybe less is better
            if (distance < 300)
            {
                //Change roll when close. When Spaceship faces target, roll doesn't affect it's trajectory
                //It's important that roll is not changed in the process of changing yaw and pitch
                //Wingmen won't face same direction as Leaders, but when Leaders start moving
                //Yaw and Pitch will adapt.
                if (bHasTargetOrientation_)
                {
                    copyTargetOrientation();
                }
            }
            this->getControllableEntity()->moveFrontBack(1.2f*SPEED);
        }
        else
        {      
            bHasTargetPosition_ = false;
            bHasTargetOrientation_ = false;
        }

      
    }
    void CommonController::moveToTargetPosition()
    {
        this->moveToPosition(this->targetPosition_);
    }
 

}
